@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003, 2004 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -172,11 +172,11 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/enhance.h"
-#include "magick/quantize.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
+#include "magick/quantize.h"
 #include "magick/utility.h"
 
 /*
@@ -185,6 +185,8 @@
 #define CacheShift  (QuantumDepth-6)
 #define ExceptionQueueLength  16
 #define MaxNodes  266817
+#define MaxTreeDepth  8
+#define NodesInAList  1536
 
 #define ColorToNodeId(red,green,blue,index) ((unsigned int) \
             (((ScaleQuantumToChar(red) >> index) & 0x01) << 2 | \
@@ -204,7 +206,7 @@ typedef struct _NodeInfo
 {
   struct _NodeInfo
     *parent,
-    *child[8];
+    *child[MaxTreeDepth];
 
   double
     number_unique;
@@ -222,14 +224,13 @@ typedef struct _NodeInfo
 
   unsigned char
     id,
-    level,
-    census;
+    level;
 } NodeInfo;
 
 typedef struct _Nodes
 {
   NodeInfo
-    nodes[NodesInAList];
+    *nodes;
 
   struct _Nodes
     *next;
@@ -344,9 +345,9 @@ static void
 %
 %
 */
-static unsigned int AssignImageColors(CubeInfo *cube_info,Image *image)
+static MagickPassFail AssignImageColors(CubeInfo *cube_info,Image *image)
 {
-#define AssignImageTag  "Assign/Image"
+#define AssignImageText  "  Assign/Image"
 
   IndexPacket
     index;
@@ -376,12 +377,15 @@ static unsigned int AssignImageColors(CubeInfo *cube_info,Image *image)
     is_grayscale,
     is_monochrome;
 
+  MagickPassFail
+    status=MagickPass;
+
   /*
     Allocate image colormap.
   */
   if (!AllocateImageColormap(image,cube_info->colors))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToQuantizeImage");
+    ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToQuantizeImage);
   image->colors=0;
   is_grayscale=image->is_grayscale;
   is_monochrome=image->is_monochrome;
@@ -399,7 +403,10 @@ static unsigned int AssignImageColors(CubeInfo *cube_info,Image *image)
     {
       q=GetImagePixels(image,0,y,image->columns,1);
       if (q == (PixelPacket *) NULL)
-        break;
+        {
+          status=MagickFail;
+          break;
+        }
       indexes=GetIndexes(image);
       for (x=0; x < (long) image->columns; x+=count)
       {
@@ -413,7 +420,7 @@ static unsigned int AssignImageColors(CubeInfo *cube_info,Image *image)
         for (index=MaxTreeDepth-1; (long) index > 0; index--)
         {
           id=ColorToNodeId(q->red,q->green,q->blue,index);
-          if ((node_info->census & (1 << id)) == 0)
+          if (node_info->child[id] == (NodeInfo *) NULL)
             break;
           node_info=node_info->child[id];
         }
@@ -440,13 +447,19 @@ static unsigned int AssignImageColors(CubeInfo *cube_info,Image *image)
         }
       }
       if (!SyncImagePixels(image))
-        break;
-      if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(AssignImageTag,y,image->rows,&image->exception))
+        {
+          status=MagickFail;
           break;
+        }
+      if (QuantumTick(y,image->rows))
+        if (!MagickMonitor(AssignImageText,y,image->rows,&image->exception))
+          {
+            status=MagickFail;
+            break;
+          }
     }
   if ((cube_info->quantize_info->number_colors == 2) &&
-      (cube_info->quantize_info->colorspace == GRAYColorspace))
+      (IsGrayColorspace(cube_info->quantize_info->colorspace)))
     {
       Quantum
         intensity;
@@ -468,10 +481,10 @@ static unsigned int AssignImageColors(CubeInfo *cube_info,Image *image)
     }
   if (cube_info->quantize_info->measure_error)
     (void) GetImageQuantizeError(image);
-  SyncImage(image);
+  status &= SyncImage(image);
   image->is_grayscale=is_grayscale;
   image->is_monochrome=is_monochrome;
-  return(True);
+  return(status);
 }
 
 /*
@@ -537,10 +550,10 @@ static unsigned int AssignImageColors(CubeInfo *cube_info,Image *image)
 %
 */
 
-static unsigned int ClassifyImageColors(CubeInfo *cube_info,const Image *image,
+static MagickPassFail ClassifyImageColors(CubeInfo *cube_info,const Image *image,
   ExceptionInfo *exception)
 {
-#define ClassifyImageTag  "Classify/Image"
+#define ClassifyImageText  "  Classify/Image"
 
   double
     bisect;
@@ -569,6 +582,9 @@ static unsigned int ClassifyImageColors(CubeInfo *cube_info,const Image *image,
   unsigned int
     id;
 
+  MagickPassFail
+    status=MagickPass;
+
   /*
     Classify the first 256 colors to a tree depth of 8.
   */
@@ -576,7 +592,10 @@ static unsigned int ClassifyImageColors(CubeInfo *cube_info,const Image *image,
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFail;
+        break;
+      }
     if (cube_info->nodes > MaxNodes)
       {
         /*
@@ -611,12 +630,11 @@ static unsigned int ClassifyImageColors(CubeInfo *cube_info,const Image *image,
             /*
               Set colors of new node to contain pixel.
             */
-            node_info->census|=(1 << id);
             node_info->child[id]=GetNodeInfo(cube_info,id,level,node_info);
             if (node_info->child[id] == (NodeInfo *) NULL)
-              ThrowException(exception,ResourceLimitError,
-                "MemoryAllocationFailed","UnableToQuantizeImage");
-            if (level == 8)
+              ThrowException3(exception,ResourceLimitError,
+                MemoryAllocationFailed,UnableToQuantizeImage);
+            if (level == MaxTreeDepth)
               cube_info->colors++;
           }
         /*
@@ -641,8 +659,11 @@ static unsigned int ClassifyImageColors(CubeInfo *cube_info,const Image *image,
       p+=count;
     }
     if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(ClassifyImageTag,y,image->rows,exception))
-        break;
+      if (!MagickMonitor(ClassifyImageText,y,image->rows,exception))
+        {
+          status=MagickFail;
+          break;
+        }
   }
   if (y == (long) image->rows)
     return(True);
@@ -654,7 +675,10 @@ static unsigned int ClassifyImageColors(CubeInfo *cube_info,const Image *image,
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFail;
+        break;
+      }
     if (cube_info->nodes > MaxNodes)
       {
         /*
@@ -689,11 +713,10 @@ static unsigned int ClassifyImageColors(CubeInfo *cube_info,const Image *image,
             /*
               Set colors of new node to contain pixel.
             */
-            node_info->census|=(1 << id);
             node_info->child[id]=GetNodeInfo(cube_info,id,level,node_info);
             if (node_info->child[id] == (NodeInfo *) NULL)
-              ThrowException(exception,ResourceLimitError,
-                "MemoryAllocationFailed","UnableToQuantizeImage");
+              ThrowException3(exception,ResourceLimitError,
+                MemoryAllocationFailed,UnableToQuantizeImage);
             if (level == cube_info->depth)
               cube_info->colors++;
           }
@@ -719,10 +742,13 @@ static unsigned int ClassifyImageColors(CubeInfo *cube_info,const Image *image,
       p+=count;
     }
     if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(ClassifyImageTag,y,image->rows,exception))
-        break;
+      if (!MagickMonitor(ClassifyImageText,y,image->rows,exception))
+        {
+          status=MagickFail;
+          break;
+        }
   }
-  return(True);
+  return(status);
 }
 
 /*
@@ -757,10 +783,10 @@ MagickExport QuantizeInfo *CloneQuantizeInfo(const QuantizeInfo *quantize_info)
   QuantizeInfo
     *clone_info;
 
-  clone_info=(QuantizeInfo *) AcquireMemory(sizeof(QuantizeInfo));
+  clone_info=MagickAllocateMemory(QuantizeInfo *,sizeof(QuantizeInfo));
   if (clone_info == (QuantizeInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToAllocateQuantizeInfo");
+    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+      UnableToAllocateQuantizeInfo);
   GetQuantizeInfo(clone_info);
   if (quantize_info == (QuantizeInfo *) NULL)
     return(clone_info);
@@ -811,10 +837,9 @@ static void ClosestColor(Image *image,CubeInfo *cube_info,
   /*
     Traverse any children.
   */
-  if (node_info->census != 0)
-    for (id=0; id < MaxTreeDepth; id++)
-      if (node_info->census & (1 << id))
-        ClosestColor(image,cube_info,node_info->child[id]);
+  for (id=0; id < MaxTreeDepth; id++)
+    if (node_info->child[id] != (NodeInfo *) NULL)
+      ClosestColor(image,cube_info,node_info->child[id]);
   if (node_info->number_unique != 0)
     {
       double
@@ -885,7 +910,7 @@ MagickExport void CompressImageColormap(Image *image)
     return;
   GetQuantizeInfo(&quantize_info);
   quantize_info.number_colors=image->colors;
-  quantize_info.tree_depth=8;
+  quantize_info.tree_depth=MaxTreeDepth;
   (void) QuantizeImage(&quantize_info,image);
 }
 
@@ -925,10 +950,9 @@ static void DefineImageColormap(Image *image,NodeInfo *node_info)
   /*
     Traverse any children.
   */
-  if (node_info->census != 0)
-    for (id=0; id < MaxTreeDepth; id++)
-      if (node_info->census & (1 << id))
-        DefineImageColormap(image,node_info->child[id]);
+  for (id=0; id < MaxTreeDepth; id++)
+    if (node_info->child[id] != (NodeInfo *) NULL)
+      DefineImageColormap(image,node_info->child[id]);
   if (node_info->number_unique != 0)
     {
       register double
@@ -982,12 +1006,13 @@ static void DestroyCubeInfo(CubeInfo *cube_info)
   do
   {
     nodes=cube_info->node_queue->next;
-    LiberateMemory((void **) &cube_info->node_queue);
+    MagickFreeMemory(cube_info->node_queue->nodes);
+    MagickFreeMemory(cube_info->node_queue);
     cube_info->node_queue=nodes;
   } while (cube_info->node_queue != (Nodes *) NULL);
   if (cube_info->quantize_info->dither)
-    LiberateMemory((void **) &cube_info->cache);
-  LiberateMemory((void **) &cube_info);
+    MagickFreeMemory(cube_info->cache);
+  MagickFreeMemory(cube_info);
 }
 
 /*
@@ -1018,7 +1043,7 @@ MagickExport void DestroyQuantizeInfo(QuantizeInfo *quantize_info)
 {
   assert(quantize_info != (QuantizeInfo *) NULL);
   assert(quantize_info->signature == MagickSignature);
-  LiberateMemory((void **) &quantize_info);
+  MagickFreeMemory(quantize_info);
 }
 
 /*
@@ -1052,7 +1077,7 @@ MagickExport void DestroyQuantizeInfo(QuantizeInfo *quantize_info)
 %      to move to next to follow the Hilbert curve.
 %
 */
-static unsigned int Dither(CubeInfo *cube_info,Image *image,
+static MagickPassFail Dither(CubeInfo *cube_info,Image *image,
   const unsigned int direction)
 {
   DoublePixelPacket
@@ -1085,7 +1110,7 @@ static unsigned int Dither(CubeInfo *cube_info,Image *image,
       */
       q=GetImagePixels(image,p->x,p->y,1,1);
       if (q == (PixelPacket *) NULL)
-        return(False);
+        return(MagickFail);
       indexes=GetIndexes(image);
       error.red=q->red;
       error.green=q->green;
@@ -1118,7 +1143,7 @@ static unsigned int Dither(CubeInfo *cube_info,Image *image,
           for (index=MaxTreeDepth-1; (long) index > 0; index--)
           {
             id=ColorToNodeId(pixel.red,pixel.green,pixel.blue,index);
-            if ((node_info->census & (1 << id)) == 0)
+            if (node_info->child[id] == (NodeInfo *) NULL)
               break;
             node_info=node_info->child[id];
           }
@@ -1145,7 +1170,7 @@ static unsigned int Dither(CubeInfo *cube_info,Image *image,
           q->blue=image->colormap[index].blue;
         }
       if (!SyncImagePixels(image))
-        return(False);
+        return(MagickFail);
       /*
         Propagate the error as the last entry of the error queue.
       */
@@ -1162,7 +1187,7 @@ static unsigned int Dither(CubeInfo *cube_info,Image *image,
     case NorthGravity: p->y--; break;
     case SouthGravity: p->y++; break;
   }
-  return(True);
+  return(MagickPass);
 }
 
 /*
@@ -1196,9 +1221,9 @@ static unsigned int Dither(CubeInfo *cube_info,Image *image,
 %
 %
 */
-static unsigned int DitherImage(CubeInfo *cube_info,Image *image)
+static MagickPassFail DitherImage(CubeInfo *cube_info,Image *image)
 {
-  register long
+  register unsigned long
     i;
 
   unsigned long
@@ -1218,12 +1243,12 @@ static unsigned int DitherImage(CubeInfo *cube_info,Image *image)
   */
   cube_info->x=0;
   cube_info->y=0;
-  i=(long) (image->columns > image->rows ? image->columns : image->rows);
+  i=image->columns > image->rows ? image->columns : image->rows;
   for (depth=1; i != 0; depth++)
     i>>=1;
   HilbertCurve(cube_info,image,depth-1,NorthGravity);
   (void) Dither(cube_info,image,ForgetGravity);
-  return(True);
+  return(MagickPass);
 }
 
 /*
@@ -1277,7 +1302,7 @@ static CubeInfo *GetCubeInfo(const QuantizeInfo *quantize_info,
   /*
     Initialize tree to describe color cube_info.
   */
-  cube_info=(CubeInfo *) AcquireMemory(sizeof(CubeInfo));
+  cube_info=MagickAllocateMemory(CubeInfo *,sizeof(CubeInfo));
   if (cube_info == (CubeInfo *) NULL)
     return((CubeInfo *) NULL);
   (void) memset(cube_info,0,sizeof(CubeInfo));
@@ -1299,7 +1324,7 @@ static CubeInfo *GetCubeInfo(const QuantizeInfo *quantize_info,
   /*
     Initialize dither resources.
   */
-  cube_info->cache=(long *) AcquireMemory((1 << 18)*sizeof(long));
+  cube_info->cache=MagickAllocateMemory(long *,(1 << 18)*sizeof(long));
   if (cube_info->cache == (long *) NULL)
     return((CubeInfo *) NULL);
   /*
@@ -1375,9 +1400,12 @@ static NodeInfo *GetNodeInfo(CubeInfo *cube_info,const unsigned int id,
       /*
         Allocate a new nodes of nodes.
       */
-      nodes=(Nodes *) AcquireMemory(sizeof(Nodes));
+      nodes=MagickAllocateMemory(Nodes *,sizeof(Nodes));
       if (nodes == (Nodes *) NULL)
         return((NodeInfo *) NULL);
+      nodes->nodes=MagickAllocateMemory(NodeInfo *,(NodesInAList*sizeof(NodeInfo)));
+      if (nodes->nodes == (NodeInfo *) NULL)
+        return((NodeInfo *) NULL);
       nodes->next=cube_info->node_queue;
       cube_info->node_queue=nodes;
       cube_info->next_node=nodes->nodes;
@@ -1435,7 +1463,7 @@ static NodeInfo *GetNodeInfo(CubeInfo *cube_info,const unsigned int id,
 %
 %
 */
-MagickExport unsigned int GetImageQuantizeError(Image *image)
+MagickExport MagickPassFail GetImageQuantizeError(Image *image)
 {
   double
     distance,
@@ -1463,15 +1491,18 @@ MagickExport unsigned int GetImageQuantizeError(Image *image)
   register long
     x;
 
+  MagickPassFail
+    status=MagickPass;
+
   /*
     Initialize measurement.
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   image->total_colors=GetNumberColors(image,(FILE *) NULL,&image->exception);
-  memset(&image->error,0,sizeof(ErrorInfo));
+  (void) memset(&image->error,0,sizeof(ErrorInfo));
   if (image->storage_class == DirectClass)
-    return(True);
+    return(MagickFail);
   /*
     For each pixel, collect error statistics.
   */
@@ -1481,7 +1512,10 @@ MagickExport unsigned int GetImageQuantizeError(Image *image)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFail;
+        break;
+      }
     indexes=GetIndexes(image);
     for (x=0; x < (long) image->columns; x++)
     {
@@ -1505,7 +1539,7 @@ MagickExport unsigned int GetImageQuantizeError(Image *image)
   image->error.normalized_mean_error=
     image->error.mean_error_per_pixel/normalize;
   image->error.normalized_maximum_error=maximum_error_per_pixel/normalize;
-  return(True);
+  return(status);
 }
 
 /*
@@ -1546,6 +1580,263 @@ MagickExport void GetQuantizeInfo(QuantizeInfo *quantize_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G r a y s c a l e P s e u d o C l a s s I m a g e                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GrayscalePseudoClassImage converts an image to a PseudoClass
+%  grayscale representation with an (optionally) compressed and sorted
+%  colormap. Colormap is ordered by increasing intensity.
+%
+%  The format of the GrayscalePseudoClassImage method is:
+%
+%      void GrayscalePseudoClassImage(Image *image)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o optimize_colormap: If true, produce an optimimal (compact) colormap.
+%
+*/
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+static int IntensityCompare(const void *x,const void *y)
+{
+  long
+    intensity;
+
+  PixelPacket
+    *color_1,
+    *color_2;
+
+  color_1=(PixelPacket *) x;
+  color_2=(PixelPacket *) y;
+  intensity=PixelIntensityToQuantum(color_1)-
+    (long) PixelIntensityToQuantum(color_2);
+  return(intensity);
+}
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+MagickExport void GrayscalePseudoClassImage(Image *image,
+  unsigned int optimize_colormap)
+{
+  long
+    y;
+
+  register long
+    x;
+
+  register IndexPacket
+    *indexes;
+
+  register const PixelPacket
+    *q;
+
+  register unsigned int
+    i;
+
+  int
+    *colormap_index=(int *) NULL;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+
+  if (!image->is_grayscale)
+    (void) TransformColorspace(image,GRAYColorspace);
+
+  if (image->storage_class != PseudoClass)
+    {
+      /*
+        Allocate maximum sized grayscale image colormap
+      */
+      if (!AllocateImageColormap(image,MaxColormapSize))
+        {
+          ThrowException3(&image->exception,ResourceLimitError,
+            MemoryAllocationFailed,UnableToSortImageColormap);
+          return;
+        }
+
+      if (optimize_colormap)
+        {
+          /*
+            Use minimal colormap method.
+          */
+
+          /*
+            Allocate memory for colormap index
+          */
+          colormap_index=MagickAllocateMemory(int *,MaxColormapSize*sizeof(int *));
+          if (colormap_index == (int *) NULL)
+            {
+              ThrowException3(&image->exception,ResourceLimitError,
+                MemoryAllocationFailed,UnableToSortImageColormap);
+              return;
+            }
+
+          /*
+            Initial colormap index value is -1 so we can tell if it
+            is initialized.
+          */
+          for (i=0; i < MaxColormapSize; i++)
+            colormap_index[i]=-1;
+
+          image->colors=0;
+          for (y=0; y < (long) image->rows; y++)
+            {
+              q=GetImagePixels(image,0,y,image->columns,1);
+              if (q == (PixelPacket *) NULL)
+                break;
+              indexes=GetIndexes(image);
+              for (x=(long) image->columns; x > 0; x--)
+                {
+                  register int
+                    intensity;
+                  
+                  /*
+                    If index is new, create index to colormap
+                  */
+                  intensity=ScaleQuantumToMap(q->red);
+                  if (colormap_index[intensity] < 0)
+                    {
+                      colormap_index[intensity]=image->colors;
+                      image->colormap[image->colors]=*q;
+                      image->colors++;
+                    }
+                  *indexes++=colormap_index[intensity];
+                  q++;
+                }
+              if (!SyncImagePixels(image))
+                return;
+            }
+        }
+      else
+        {
+          /*
+            Use fast-cut linear colormap method.
+          */
+          for (y=0; y < (long) image->rows; y++)
+            {
+              q=GetImagePixels(image,0,y,image->columns,1);
+              if (q == (PixelPacket *) NULL)
+                break;
+              indexes=GetIndexes(image);
+              for (x=(long) image->columns; x > 0; x--)
+                {
+                  *indexes=ScaleQuantumToIndex(q->red);
+                  q++;
+                  indexes++;
+                } 
+              if (!SyncImagePixels(image))
+                break;
+           }
+          image->is_grayscale=True;
+          return;
+        }
+    }
+
+  if (optimize_colormap)
+    {
+      /*
+        Sort and compact the colormap
+      */
+
+      /*
+        Allocate memory for colormap index
+      */
+      if (colormap_index == (int *) NULL)
+        {
+          colormap_index=MagickAllocateMemory(int *,MaxColormapSize*sizeof(int *));
+          if (colormap_index == (int *) NULL)
+            {
+              ThrowException3(&image->exception,ResourceLimitError,
+                MemoryAllocationFailed,UnableToSortImageColormap);
+              return;
+            }
+        }
+      
+      /*
+        Assign index values to colormap entries.
+      */
+      for (i=0; i < image->colors; i++)
+        image->colormap[i].opacity=(unsigned short) i;
+      /*
+        Sort image colormap by increasing intensity.
+      */
+      qsort((void *) image->colormap,image->colors,sizeof(PixelPacket),
+            IntensityCompare);
+      /*
+        Create mapping between original indexes and reduced/sorted
+        colormap.
+      */
+      {
+        PixelPacket
+          *new_colormap;
+
+        int
+          j;
+
+        new_colormap=MagickAllocateMemory(PixelPacket *,image->colors*sizeof(PixelPacket));
+        if (new_colormap == (PixelPacket *) NULL)
+          {
+            ThrowException3(&image->exception,ResourceLimitError,
+              MemoryAllocationFailed,UnableToSortImageColormap);
+            return;
+          }
+
+        j=0;
+        new_colormap[j]=image->colormap[0];
+        for (i=0; i < image->colors; i++)
+          {
+            if (!ColorMatch(&new_colormap[j],&image->colormap[i]))
+              {
+                j++;
+                new_colormap[j]=image->colormap[i];
+              }
+            
+            colormap_index[image->colormap[i].opacity]=j;
+          }
+        image->colors=j+1;
+        MagickFreeMemory(image->colormap);
+        image->colormap=new_colormap;
+      }
+
+      /*
+        Reassign image colormap indexes
+      */
+      for (y=0; y < (long) image->rows; y++)
+        {
+          q=GetImagePixels(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
+          indexes=GetIndexes(image);
+          for (x=(long) image->columns; x > 0; x--)
+            {
+              *indexes=colormap_index[*indexes];
+              indexes++;
+            }
+          if (!SyncImagePixels(image))
+            break;
+        }
+      MagickFreeMemory(colormap_index);
+    }
+  image->is_monochrome=IsMonochromeImage(image,&image->exception);
+  image->is_grayscale=True;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   H i l b e r t C u r v e                                                   %
 %                                                                             %
 %                                                                             %
@@ -1696,7 +1987,7 @@ static void HilbertCurve(CubeInfo *cube_info,Image *image,
 %
 %
 */
-MagickExport unsigned int MapImage(Image *image,const Image *map_image,
+MagickExport MagickPassFail MapImage(Image *image,const Image *map_image,
   const unsigned int dither)
 {
   CubeInfo
@@ -1705,8 +1996,8 @@ MagickExport unsigned int MapImage(Image *image,const Image *map_image,
   QuantizeInfo
     quantize_info;
 
-  unsigned int
-    status;
+  MagickPassFail
+    status=MagickPass;
 
   /*
     Initialize color cube.
@@ -1719,12 +2010,12 @@ MagickExport unsigned int MapImage(Image *image,const Image *map_image,
   quantize_info.dither=dither;
   quantize_info.colorspace=
     image->matte ? TransparentColorspace : RGBColorspace;
-  cube_info=GetCubeInfo(&quantize_info,8);
+  cube_info=GetCubeInfo(&quantize_info,MaxTreeDepth);
   if (cube_info == (CubeInfo *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToMapImage");
+    ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToMapImage);
   status=ClassifyImageColors(cube_info,map_image,&image->exception);
-  if (status != False)
+  if (status != MagickFail)
     {
       /*
         Classify image colors from the reference image.
@@ -1748,7 +2039,12 @@ MagickExport unsigned int MapImage(Image *image,const Image *map_image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  MapImages() replaces the colors of a sequence of images with the closest
-%  color from a reference image.
+%  color from a reference image. If the reference image does not contain a
+%  colormap, then a colormap will be created based on existing colors in the
+%  reference image. The order and number of colormap entries does not match
+%  the reference image.  If the order and number of colormap entries needs to
+%  match the reference image, then the ReplaceImageColormap() function may be
+%  used after invoking MapImages() in order to apply the reference colormap.
 %
 %  The format of the MapImage method is:
 %
@@ -1767,7 +2063,7 @@ MagickExport unsigned int MapImage(Image *image,const Image *map_image,
 %
 %
 */
-MagickExport unsigned int MapImages(Image *images,const Image *map_image,
+MagickExport MagickPassFail MapImages(Image *images,const Image *map_image,
   const unsigned int dither)
 {
   CubeInfo
@@ -1779,7 +2075,7 @@ MagickExport unsigned int MapImages(Image *images,const Image *map_image,
   QuantizeInfo
     quantize_info;
 
-  unsigned int
+  MagickPassFail
     status;
 
   assert(images != (Image *) NULL);
@@ -1803,10 +2099,10 @@ MagickExport unsigned int MapImages(Image *images,const Image *map_image,
   */
   cube_info=GetCubeInfo(&quantize_info,8);
   if (cube_info == (CubeInfo *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToMapImageSequence");
+    ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToMapImageSequence);
   status=ClassifyImageColors(cube_info,map_image,&image->exception);
-  if (status != False)
+  if (status != MagickFail)
     {
       /*
         Classify image colors from the reference image.
@@ -1817,7 +2113,7 @@ MagickExport unsigned int MapImages(Image *images,const Image *map_image,
         quantize_info.colorspace=image->matte ? TransparentColorspace :
           RGBColorspace;
         status=AssignImageColors(cube_info,image);
-        if (status == False)
+        if (status == MagickFail)
           break;
       }
     }
@@ -1851,9 +2147,9 @@ MagickExport unsigned int MapImages(Image *images,const Image *map_image,
 %
 %
 */
-MagickExport unsigned int OrderedDitherImage(Image *image)
+MagickExport MagickPassFail OrderedDitherImage(Image *image)
 {
-#define DitherImageTag  "Dither/Image"
+#define DitherImageText  "  Dither/Image"
 
   static Quantum
     DitherMatrix[8][8] =
@@ -1883,13 +2179,16 @@ MagickExport unsigned int OrderedDitherImage(Image *image)
   register PixelPacket
     *q;
 
+  MagickPassFail
+    status=MagickPass;
+
   /*
     Initialize colormap.
   */
   (void) NormalizeImage(image);
   if (!AllocateImageColormap(image,2))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToDitherImage");
+    ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToDitherImage);
   /*
     Dither image with the ordered dithering technique.
   */
@@ -1897,7 +2196,10 @@ MagickExport unsigned int OrderedDitherImage(Image *image)
   {
     q=GetImagePixels(image,0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFail;
+        break;
+      }
     indexes=GetIndexes(image);
     for (x=0; x < (long) image->columns; x++)
     {
@@ -1910,12 +2212,18 @@ MagickExport unsigned int OrderedDitherImage(Image *image)
       q++;
     }
     if (!SyncImagePixels(image))
-      break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(DitherImageTag,y,image->rows,&image->exception))
+      {
+        status=MagickFail;
         break;
+      }
+    if (QuantumTick(y,image->rows))
+      if (!MagickMonitor(DitherImageText,y,image->rows,&image->exception))
+        {
+          status=MagickFail;
+          break;
+        }
   }
-  return(True);
+  return(status);
 }
 
 /*
@@ -1955,19 +2263,18 @@ static void PruneChild(CubeInfo *cube_info,const NodeInfo *node_info)
   /*
     Traverse any children.
   */
-  if (node_info->census != 0)
-    for (id=0; id < MaxTreeDepth; id++)
-      if (node_info->census & (1 << id))
-        PruneChild(cube_info,node_info->child[id]);
+  for (id=0; id < MaxTreeDepth; id++)
+    if (node_info->child[id] != (NodeInfo *) NULL)
+      PruneChild(cube_info,node_info->child[id]);
   /*
     Merge color statistics into parent.
   */
   parent=node_info->parent;
-  parent->census&=~(1 << node_info->id);
   parent->number_unique+=node_info->number_unique;
   parent->total_red+=node_info->total_red;
   parent->total_green+=node_info->total_green;
   parent->total_blue+=node_info->total_blue;
+  parent->child[node_info->id]=(NodeInfo *) NULL;
   cube_info->nodes--;
 }
 
@@ -2005,10 +2312,9 @@ static void PruneLevel(CubeInfo *cube_info,const NodeInfo *node_info)
   /*
     Traverse any children.
   */
-  if (node_info->census != 0)
-    for (id=0; id < MaxTreeDepth; id++)
-      if (node_info->census & (1 << id))
-        PruneLevel(cube_info,node_info->child[id]);
+  for (id=0; id < MaxTreeDepth; id++)
+    if (node_info->child[id] != (NodeInfo *) NULL)
+      PruneLevel(cube_info,node_info->child[id]);
   if (node_info->level == cube_info->depth)
     PruneChild(cube_info,node_info);
 }
@@ -2048,10 +2354,9 @@ static void PruneToCubeDepth(CubeInfo *cube_info,const NodeInfo *node_info)
   /*
     Traverse any children.
   */
-  if (node_info->census != 0)
-    for (id=0; id < MaxTreeDepth; id++)
-      if (node_info->census & (1 << id))
-        PruneToCubeDepth(cube_info,node_info->child[id]);
+  for (id=0; id < MaxTreeDepth; id++)
+    if (node_info->child[id] != (NodeInfo *) NULL)
+      PruneToCubeDepth(cube_info,node_info->child[id]);
   if (node_info->level > cube_info->depth)
     PruneChild(cube_info,node_info);
 }
@@ -2084,13 +2389,13 @@ static void PruneToCubeDepth(CubeInfo *cube_info,const NodeInfo *node_info)
 %    o image: Specifies a pointer to an Image structure.
 %
 */
-MagickExport unsigned int QuantizeImage(const QuantizeInfo *quantize_info,
+MagickExport MagickPassFail QuantizeImage(const QuantizeInfo *quantize_info,
   Image *image)
 {
   CubeInfo
     *cube_info;
 
-  unsigned int
+  MagickPassFail
     status;
 
   unsigned long
@@ -2111,8 +2416,8 @@ MagickExport unsigned int QuantizeImage(const QuantizeInfo *quantize_info,
     which assures that the maximum number of colors is equal to, or
     less than MaxColormapSize.
   */
-  if (quantize_info->colorspace == GRAYColorspace)
-    TransformColorspace(image,quantize_info->colorspace);
+  if (IsGrayColorspace(quantize_info->colorspace))
+    (void) TransformColorspace(image,quantize_info->colorspace);
   if (IsGrayImage(image,&image->exception))
     GrayscalePseudoClassImage(image,True);
   /*
@@ -2121,7 +2426,7 @@ MagickExport unsigned int QuantizeImage(const QuantizeInfo *quantize_info,
   */
   if ((image->storage_class == PseudoClass) &&
       (image->colors <= number_colors))
-    return(True);
+    return(MagickPass);
   depth=quantize_info->tree_depth;
   if (depth == 0)
     {
@@ -2144,12 +2449,12 @@ MagickExport unsigned int QuantizeImage(const QuantizeInfo *quantize_info,
   */
   cube_info=GetCubeInfo(quantize_info,depth);
   if (cube_info == (CubeInfo *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToQuantizeImage");
+    ThrowBinaryException3(ResourceLimitError,
+      MemoryAllocationFailed,UnableToQuantizeImage);
   if (quantize_info->colorspace != RGBColorspace)
-    TransformColorspace(image,quantize_info->colorspace);
+    (void) TransformColorspace(image,quantize_info->colorspace);
   status=ClassifyImageColors(cube_info,image,&image->exception);
-  if (status != False)
+  if (status != MagickFail)
     {
       /*
         Reduce the number of colors in the image.
@@ -2157,7 +2462,7 @@ MagickExport unsigned int QuantizeImage(const QuantizeInfo *quantize_info,
       ReduceImageColors(cube_info,number_colors,&image->exception);
       status=AssignImageColors(cube_info,image);
       if (quantize_info->colorspace != RGBColorspace)
-        TransformColorspace(image,quantize_info->colorspace);
+        (void) TransformColorspace(image,quantize_info->colorspace);
     }
   DestroyCubeInfo(cube_info);
   return(status);
@@ -2192,7 +2497,7 @@ MagickExport unsigned int QuantizeImage(const QuantizeInfo *quantize_info,
 %
 %
 */
-MagickExport unsigned int QuantizeImages(const QuantizeInfo *quantize_info,
+MagickExport MagickPassFail QuantizeImages(const QuantizeInfo *quantize_info,
   Image *images)
 {
   CubeInfo
@@ -2264,13 +2569,13 @@ MagickExport unsigned int QuantizeImages(const QuantizeInfo *quantize_info,
   */
   cube_info=GetCubeInfo(quantize_info,depth);
   if (cube_info == (CubeInfo *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToQuantizeImageSequence");
+    ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToQuantizeImageSequence);
   image=images;
   for (i=0; image != (Image *) NULL; i++)
   {
     if (quantize_info->colorspace != RGBColorspace)
-      TransformColorspace(image,quantize_info->colorspace);
+      (void) TransformColorspace(image,quantize_info->colorspace);
     image=image->next;
   }
   number_images=i;
@@ -2279,14 +2584,14 @@ MagickExport unsigned int QuantizeImages(const QuantizeInfo *quantize_info,
   {
     handler=SetMonitorHandler((MonitorHandler) NULL);
     status=ClassifyImageColors(cube_info,image,&image->exception);
-    if (status == False)
+    if (status == MagickFail)
       break;
     image=image->next;
     (void) SetMonitorHandler(handler);
-    if (!MagickMonitor(ClassifyImageTag,i,number_images,&image->exception))
+    if (!MagickMonitor(ClassifyImageText,i,number_images,&image->exception))
       break;
   }
-  if (status != False)
+  if (status != MagickFail)
     {
       /*
         Reduce the number of colors in an image sequence.
@@ -2297,14 +2602,17 @@ MagickExport unsigned int QuantizeImages(const QuantizeInfo *quantize_info,
       {
         handler=SetMonitorHandler((MonitorHandler) NULL);
         status=AssignImageColors(cube_info,image);
-        if (status == False)
+        if (status == MagickFail)
           break;
         if (quantize_info->colorspace != RGBColorspace)
-          TransformColorspace(image,quantize_info->colorspace);
+          (void) TransformColorspace(image,quantize_info->colorspace);
         image=image->next;
         (void) SetMonitorHandler(handler);
-        if (!MagickMonitor(AssignImageTag,i,number_images,&image->exception))
-          break;
+        if (!MagickMonitor(AssignImageText,i,number_images,&image->exception))
+          {
+            status=MagickFail;
+            break;
+          }
       }
     }
   DestroyCubeInfo(cube_info);
@@ -2345,10 +2653,9 @@ static void Reduce(CubeInfo *cube_info,const NodeInfo *node_info)
   /*
     Traverse any children.
   */
-  if (node_info->census != 0)
-    for (id=0; id < MaxTreeDepth; id++)
-      if (node_info->census & (1 << id))
-        Reduce(cube_info,node_info->child[id]);
+  for (id=0; id < MaxTreeDepth; id++)
+    if (node_info->child[id] != (NodeInfo *) NULL)
+      Reduce(cube_info,node_info->child[id]);
   if (node_info->quantize_error <= cube_info->pruning_threshold)
     PruneChild(cube_info,node_info);
   else
@@ -2426,7 +2733,7 @@ static void Reduce(CubeInfo *cube_info,const NodeInfo *node_info)
 static void ReduceImageColors(CubeInfo *cube_info,
   const unsigned long number_colors,ExceptionInfo *exception)
 {
-#define ReduceImageTag  " Reduce/Image"
+#define ReduceImageText  " Reduce/Image"
 
   unsigned int
     status;
@@ -2442,7 +2749,7 @@ static void ReduceImageColors(CubeInfo *cube_info,
     cube_info->next_threshold=cube_info->root->quantize_error-1;
     cube_info->colors=0;
     Reduce(cube_info,cube_info->root);
-    status=MagickMonitor(ReduceImageTag,span-cube_info->colors,
+    status=MagickMonitor(ReduceImageText,span-cube_info->colors,
       span-number_colors+1,exception);
     if (status == False)
       break;
